//
//  GzViewController.m
//  Twitter Search Example
//
//  Created by Geenz on 7/18/13.
//  Copyright (c) 2013 Geenz. All rights reserved.
//

#import "GzViewController.h"
#import "TweetViewCell.h"
#import "TweetSearchBar.h"

@interface GzViewController ()

@end

@implementation GzViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
    
    // Set our API URL.
    self.apiURL = [NSURL URLWithString:@"https://api.twitter.com/1.1/"]; // We're using specifically 1.1 of the twitter REST APIs here.
    
    // Grab the system's account store, and check if we have a twitter account associated with the device.
    self.accountStore = [[ACAccountStore alloc]init];
    if (![self userHasAccessToTwitter])
    {
        [self displayNoTwitterError];
        // In theory, the app would quit after this.  But in keeping with Apple's approval process, I'll disclude that (unless something's changed in that respect).
    } else {
        NSLog(@"We have twitter!\n");
    }
    
    self.searchTerm = @"";
    
    // Setup our refresh control.
    [self.refreshControl addTarget:self action:@selector(refreshSearchResults) forControlEvents:UIControlEventValueChanged];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
    [self cancelConnection];
    
    // Typically, you'd make a call to [self.results release] here to free up self.results.
    // But this application makes use of automatic reference counting, which means there's no need to manually release and retain objects, or to manually call dealloc on objects.
    // This is both more performant than potentially calling release more than necessary, and is also less accident prone than manually managing your memory.
    self.results = nil;
}

- (IBAction)refreshSearchResults
{
    if (self.results)
    {
        [self cancelConnection];
        [self requestTwitterSearch:self.searchTerm];
    } else {
        [self.refreshControl endRefreshing];
    }
}

/******************************
 * These are largely just helper functions.
 * cancelConnection was taken verbatim from https://github.com/kharrison/CodeExamples/tree/master/TwitterSearch , as was userHasAccessToTwitter.
 * displayNoTwitterError is just a helper to give the user feedback if they either haven't authenticated their twitter account, or don't have an account associated with their device.
 ******************************/

- (void)cancelConnection
{
    if (self.connection != nil)
    {
        [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
        [self.connection cancel];
        self.connection = nil;
        self.requestData = nil;
    }
}

- (BOOL)userHasAccessToTwitter
{
    return [SLComposeViewController isAvailableForServiceType:SLServiceTypeTwitter];
}

- (void) displayNoTwitterError
{
    // Some users may not have a twitter account associated with their devices.
    // Sadly, Twitter only allows authenticated accounts to access their REST APIs now days, even just for using the search API.  Alert the user to the problem.
    UIAlertView* alert = [[UIAlertView alloc] initWithTitle:@"No Twitter account found!" message:@"Twitter requires an authetnicated account in order to use their APIs.  Please add an account to your device to continue." delegate:nil cancelButtonTitle:@"Okay" otherButtonTitles:nil];
    [alert show];
}

/******************************
 * requestTwitterSearch creates a request to a specified twitter service URL.  In this case, https://api.twitter.com/1.1/search/tweets.json
 * From there, NSURLConnection takes over, calling the relevant delegates below.
 ******************************/

- (void) requestTwitterSearch:(NSString*)term
{
    if (![term isEqualToString:@""]) {
        
        ACAccountType *twitterAccountType = [self.accountStore accountTypeWithAccountTypeIdentifier: ACAccountTypeIdentifierTwitter];
        
        // Whatever happened to the days when we could just search without authenticated accounts? :(
        [self.accountStore requestAccessToAccountsWithType:twitterAccountType options:NULL completion:^(BOOL granted, NSError *error) {
            
            // Per the official Twitter API documentation, 'q' is the only required parameter here, and contains our search query.
            NSDictionary *params = @{@"q" : term};
            
            // Generalized solution to interacting with the Twitter REST APIs: append the API URL as needed for whichever API we need.
            // Also, issue this request as a GET.
            SLRequest *request = [SLRequest requestForServiceType:SLServiceTypeTwitter requestMethod:SLRequestMethodGET URL:[self.apiURL URLByAppendingPathComponent:@"search/tweets.json"] parameters:params];
            
            // Any Twitter account will do.
            NSArray *twitterAccounts = [self.accountStore accountsWithAccountType:twitterAccountType];
            request.account = twitterAccounts.lastObject;
            
            // Paranoia: dispatch this asynchronously to prevent potentially blocking the main thread.
            dispatch_async(dispatch_get_main_queue(), ^{
                self.connection = [[NSURLConnection alloc] initWithRequest:[request preparedURLRequest] delegate:self];
                [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
            });
        }];
    }
}

/******************************
 * The search bar delegates.
 * These are called when data is entered into the search bar, the search button is clicked, the cancel button is clicked, or if it's changing its editing state.
 * These are all defined in the UISearchBarDelegate protocol, and implemented here.
 ******************************/

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText
{
    self.searchTerm = searchText;
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar
{
    // A user can add (or remove) their account at will in iOS 6 or 7.  Doing so will cause the app to fail if we make an attempt to request data.  Notify them of this.
    if ([self userHasAccessToTwitter])
    {
        [self requestTwitterSearch:self.searchTerm];
    } else {
        [self displayNoTwitterError];
    }
    
    // When we're not attempting to search, have the search bar resign being the first responder to make the keyboard go away.
    [searchBar resignFirstResponder];
}

- (void)searchBarCancelButtonClicked:(UISearchBar *) searchBar
{
    // Hide the cancel button.
    [searchBar setShowsCancelButton:NO animated:YES];
    
    // When we're not attempting to search, have the search bar resign being the first responder to make the keyboard go away.
    [searchBar resignFirstResponder];
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar {
    // Show the cancel button.
    [searchBar setShowsCancelButton:YES animated:YES];
    
    return YES;
}

- (BOOL) searchBarShouldEndEditing:(UISearchBar *)searchBar
{
    [searchBar setShowsCancelButton:NO animated:YES];
    
    return YES;
}

/******************************
 * The NSURLConnection delegates.
 * These are responsible for recieving data from twitter.
 * When we recieve the initial connection data, we initialize requestData.
 * Each subsequent time we recieve data, we append that with the new data.
 * Once we've recieved all of the data, we propagate the array of tweet information.
 * If the connection fails before we completely recieve all of the data for whatever reason, we wipe out any data we recieved previously.
 * These are all defined in the NSURLConnectionDelegate protocol, and implemented here.
 ******************************/

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    self.requestData = [NSMutableData data];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    [[self requestData] appendData:data];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    self.connection = nil;
    
    // Once we've finished loading the data, assign it all to an array.
    if (self.requestData) {
        NSError *jsonError;
        NSDictionary* dict = [NSJSONSerialization JSONObjectWithData:self.requestData
                                                             options:NSJSONReadingAllowFragments
                                                               error:&jsonError];
        self.results = dict[@"statuses"];
    }
    
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
    self.requestData = nil;
    [self.refreshControl endRefreshing];
    
    // Ensure that our table view gets the new data.
    [self.tableView reloadData];
    [self.tableView flashScrollIndicators];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
    self.connection = nil;
    self.requestData = nil;
    [self.refreshControl endRefreshing];
    
    [self.tableView reloadData];
}

/******************************
 * The table view delegates.
 * These are responsible for propagating the table view with tweet data.
 * This could probably be generalized a bit, instead passing tweet data directly into the tweet cells and letting those cells sort through the data.
 * These are all defined as part of UITableViewController, and implemented here.
 ******************************/

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSUInteger count = [self.results count];
    return count > 0 ? count : 1;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *ResultCellIdentifier = @"ResultCell";
    static NSString *LoadCellIdentifier = @"LoadingCell";
    
    // The user probably hasn't submitted a search term yet.  Notify them.
    NSUInteger count = [self.results count];
    if ((count == 0) && (indexPath.row == 0))
    {
        TweetViewCell *cell = [tableView dequeueReusableCellWithIdentifier:LoadCellIdentifier];
        return cell;
    }
    
    // If we've gotten to this point, that means we have data that we need to display.
    TweetViewCell *cell = [tableView dequeueReusableCellWithIdentifier:ResultCellIdentifier];
    
    // Each element of the results array is a dictionary.
    // This gives us a lot of information about the tweet, such as the user's display name, the tweet's data, and the user's profile picture.
    // All pretty useful stuff.
    NSDictionary *tweet = (self.results)[indexPath.row];
    [cell propagateTweet:tweet[@"text"]];
    [cell propagateUserName:(tweet[@"user"])[@"screen_name"]];
    
    // We may get a lot of images at once, so it's best to leverage GCD to asynchronously pull in profile images.
    dispatch_async(dispatch_get_main_queue(), ^{
        [cell propagateUserImage:[NSURL URLWithString:(tweet[@"user"])[@"profile_image_url"]]];
    });
    
    return cell;
}

@end
